
/*
 * Types/enumerations for [Atom] DBO investigation programs
 *
 *	Copyright (C) 1999, Rick Gorton
 *
 *	Written by Rick Gorton, 1999
 *	gorton@world.std.com
 *
 * This ATOM tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This ATOM tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this ATOM tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * NOTE:
 *	If you find this tool useful, the author is interested in knowing
 *	how you used it, and what kind of modifications you found useful.
 *
 */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

typedef unsigned char	U8;
typedef unsigned char	u8;
typedef u8	BOOLEAN;

typedef union {
		long	longstuff;
		double	doublestuff;
		struct {
			unsigned long	mantissa:52;
			unsigned long	exponent:11;
			unsigned long	sign:1;
			} rawbits;
		} IEEE_UNION_;

enum FloatPrecision_Enum {
	UNINITIALIZED_PREC_STATE,
	SINGLE_PREC_STATE,
	DOUBLE_PREC_STATE,
	ERROR_PREC_STATE
	};

#define	NUM_FP_REGS_	32
enum FloatPrecision_Enum	FP_RegState[NUM_FP_REGS_];

typedef struct {
	enum FloatPrecision_Enum	curr_state;
	enum Opcode			last_set_opcode;
	long				insn_address;
	long				mem_addr;
	long				seq_num;
	} FP_STATE_TYPE;

FP_STATE_TYPE	FP_State[NUM_FP_REGS_];

#define PARAM_TABLE_TYPE_SIZE	16
typedef struct {
	IEEE_UNION_	data_val[PARAM_TABLE_TYPE_SIZE];
	long		hits[PARAM_TABLE_TYPE_SIZE];
	IEEE_UNION_	delta_from_prior;
	long		delta_hits;
	long		delta_restarts;
	} PARAM_TABLE_TYPE;

#define	PROC_TABLE_TYPE_SIZE	10000
#define	PROC_NAME_SIZE		64
typedef struct {
	char		ProcName[PROC_NAME_SIZE];
	long		StartAddr;
	long		counter;
	} PROC_TABLE_TYPE;

typedef struct {
	union {
	    long	block_id_long;
	    int		block_id_int[2];
	    } u;
	void	*next_block;		/* actually WHOLE_PATH_BLOCK_ELT */
	} WHOLE_PATH_BLOCK_ELT;

	/*
	 * sizeof (PATH_LIST_TYPE) == 32 bytes
	 * sizeof (TEMPORAL_ELT) == 24 bytes
	 */
typedef struct PATH_LIST_TYPE {
	WHOLE_PATH_BLOCK_ELT	*path_chain;
	int			path_length;
	int			last_block_id;
	int			counter;
	int			seq_num;
	void			*next_path;	/* actually PATH_LIST_TYPE */
	} PATH_LIST_TYPE;

typedef struct TEMPORAL_ELT {
	int		path_seq_num;
	int		sequential_count;
	PATH_LIST_TYPE	*path_p;
	void		*next;
	} TEMPORAL_ELT;

	/* For most programs, 50000 is sufficient.  126.gcc needs more */
	/* PovRay needs more than gcc */
#define	BLOCK_TABLE_TYPE_SIZE	150000
typedef struct {
	long	ProcId;
	long	Num_Insns;
	long	Block_Addr;
	} BLOCK_TABLE_TYPE;

	/*
	 * For simple block node counts
	 */
typedef struct {
	long	counter;
	} NODE_TABLE_TYPE;

/*
 * States for replay trap detection
 */
enum MemoryState {
	LOAD_STATE,
	STORE_STATE,
	MEM_ERROR_STATE
	};

/*
 * Structures for Atom-izing time data collection
 */
static long ProcSequenceNum;
static PROC_TABLE_TYPE ProcTable[PROC_TABLE_TYPE_SIZE];
static long BlockSequenceNum;
static BLOCK_TABLE_TYPE BlockTable[BLOCK_TABLE_TYPE_SIZE];

/*
 * Structures for execution time data collection
 */

FILE	*Inst_Logfile;

/*
 * Macros (YUK) and constants for extracting the LIT field from an
 * operate instruction, since the ATOM people didn't deem it important
 * enough to provide this feature.  (Grumble)
 */

#define	OPCODE_MASK		0xfc000000
#define	OPERATE_MASK		0xfc001fe0
#define	JUMP_MASK		0xfc00c000
#define LITERAL_REGISTER_MASK	0x1000
#define LITERAL_FIELD_MASK	0x1fe000
#define LITERAL_SHIFT_COUNT	13
#define LITERAL_BIT_MASK	0xff

#define IS_LITERAL_OPERATE_(OPCODE32)	\
	(OPCODE32 & LITERAL_REGISTER_MASK)
#define LITERAL_VALUE_(OPCODE32)	\
	((OPCODE32 >> LITERAL_SHIFT_COUNT) & LITERAL_BIT_MASK)

