
/*
 *	Atom program to track selected (focus on) procedure path profiles
 *	Purpose:
 *		Program diagnostic tool, which prints out procedure level
 *		paths which include the procedure specified by the environment
 *		variable FPP_NAME.
 *
 *	Copyright (C) 2000, Rick Gorton
 *
 *	Written by Rick Gorton, 8-Mar-2000
 *	gorton@world.std.com
 *
 * This ATOM tool is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This ATOM tool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this ATOM tool; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * NOTE:
 *	If you find this tool useful, the author is interested in knowing
 *	how you used it, and what kind of modifications you found useful.
 *
 */

#include <stdio.h>
#include <cmplrs/atom.inst.h>
#include "datatypes.h"
#include "declare.h"

void InstrumentInit(int iargc, char **iargv)
{
  int	i,j;
  char	namebuffer[256];

  if (iargc > 1)
    strcpy(namebuffer, iargv[1]);
  else
    strcpy(namebuffer, LOGFILE_NAME_);
  Inst_Logfile = fopen(namebuffer, "a+");
  printf("Logfile will be '%s'\n", namebuffer);
  fprintf(Inst_Logfile, "Logfile:\t%s\n", namebuffer);
  fprintf (Inst_Logfile, "*** Static data begin ***\n");

  ProcSequenceNum = 1;
	/*
	 * Because of analysis-time constraints, this cannot be 0.
	 * which means we need to have loops using ProcSequenceNum
	 * start at 1 instead of 0.
	 */

  AddCallProto ("Extend_Path(long)");

  AddCallProto("DynamicInit(char *)");
  AddCallProgram(ProgramBefore, "DynamicInit", namebuffer);
}

Instrument(int argc, char **argv, Obj *obj)
{
  Proc *p; Block *b;  Inst *this_inst;
  RegvType	RegA, RegB, RegC;
  long		this_PC;
  enum Opcodes	curr_opcode;

  fprintf(Inst_Logfile, "Object:\t%s\n", GetObjName(obj));
  fprintf(Inst_Logfile, "Number of Procedures:\t%d\n", GetObjInfo(obj, ObjNumberProcs));

  for (p = GetFirstObjProc(obj); p != NULL; p = GetNextProc(p))
    {
	/*
	 * Build ProcTable up so we can pass its contents to the runtime
	 * _after_ we allocate space for it.
	 */
    if (ProcName(p) == NULL)
	{
	sprintf (&ProcTable[ProcSequenceNum].ProcName[0], "0x%lx", ProcPC(p));
	}
    else
	strncat (&ProcTable[ProcSequenceNum].ProcName[0], ProcName(p), PROC_NAME_SIZE);
    ProcTable[ProcSequenceNum].StartAddr = ProcPC(p);

#if 0
	// Print out procedure name, id, and address
    fprintf(Inst_Logfile, "%s (%ld)\t@ 0x%lx\n",
			ProcTable[ProcSequenceNum].ProcName,
			ProcSequenceNum, ProcPC(p));
#endif
    AddCallProc (p, ProcBefore, "Extend_Path", ProcSequenceNum);

    ProcSequenceNum++;
    if (ProcSequenceNum > PROC_TABLE_TYPE_SIZE)
	{
	fprintf (stderr, "ProcSequenceNum (0x%ld) > PROC_TABLE_TYPE_SIZE (0x%ld)\n",
		ProcSequenceNum, PROC_TABLE_TYPE_SIZE);
	fprintf (stderr, "Adjust datatypes.h & rebuild\n");
	exit (1);
	}
    }	/* Each proc */
}

void InstrumentFini()
{
  long	i;
  long	netval;

  AddCallProto ("DynAllocateStorage(long)");
  AddCallProgram (ProgramBefore, "DynAllocateStorage", ProcSequenceNum);
  AddCallProto ("Dyn_ProcCountInit(long,char *,long)");

  for (i = 1; i < ProcSequenceNum; i++)
    {
    AddCallProgram (ProgramBefore, "Dyn_ProcCountInit",
			i, ProcTable[i].ProcName,
			ProcTable[i].StartAddr);
    }

  AddCallProto ("DynamicEnvVarInit()");
  AddCallProgram (ProgramBefore, "DynamicEnvVarInit");
  AddCallProto ("DynamicFini()");
  AddCallProgram (ProgramAfter, "DynamicFini");

  fprintf (Inst_Logfile, "*** Dynamic data begin ***\n");
  fclose(Inst_Logfile);

}

